//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "ContactsTableViewController.h"
#import "ContactTableViewCell.h"
#import "PCContact+Additions.h"
#import <Contacts/Contacts.h>
#import <ContactsUI/ContactsUI.h>
#import "UIViewController+RightBarButtonAction.h"
#import "UIViewController+Spinner.h"
#import "PCAccountInfo+additions.h"
#import "UIViewController+Alert.h"
#import "UIViewController+Contact.h"
#import "ContactDetailViewController.h"
#import "ImageDownloadView.h"
#import "ContactSearchViewController.h"

@interface ContactsTableViewController () <CustomTableViewCellDelegate>

@end

@implementation ContactsTableViewController


- (void)viewDidLoad {

    [super viewDidLoad];
}

#pragma mark -
#pragma mark UITableView data source method

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataSource.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    ContactTableViewCell *cell = (ContactTableViewCell *)[tableView dequeueReusableCellWithIdentifier:@"contactsCell" forIndexPath:indexPath];
    
    PCContact *contact = [self.dataSource objectAtIndex:indexPath.row];
    
    cell.mainTextLabel.text = [contact getDisplaytext]; // return firstname or lastname or orgName or email or phoneNumber which is available.
    cell.subTextlabel.text = [contact getAnyPhoneNumber]; // return phone number.
    
    // To check PCContact is marked as favorite or not.
    if (contact.favorite) {
        
        UIImageView *imageView = [[UIImageView alloc] initWithImage:[UIImage imageNamed:@"Favorite"]];
        imageView.frame = CGRectMake(0.0, 0.0, 20, 20);
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.accessoryView = imageView;
    }
    cell.delegate = self;
    
    if ((contact.photoReference == cell.avatarImageView.imageID) && cell.avatarImageView.api) {
        // do nothing
    } else {
        
        [cell.avatarImageView.api cancelAPI];
        
        [cell.spinner startAnimating];
        // This method download thumbnail image and cache it in document directory and assign it to avatarImageView.
        [contact setAvatarForImageView:cell.avatarImageView withCompletionHandler:^(BOOL didSetImage) {
            
            if (didSetImage) {
                [cell.spinner stopAnimating];
            }
        }];
    }

    
    return cell;
}

#pragma mark -
#pragma mark UITableView delegate method

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {

    PCContact *PCContact = [self.dataSource objectAtIndex:indexPath.row];
    CNMutableContact *CNcontact = [PCContact getCNContact]; // Using PCContact object to create CNMutableContact object.
    
    
    ContactDetailViewController *contactViewController = [ContactDetailViewController viewControllerForContact:CNcontact];
    contactViewController.selectedContact = PCContact;
    contactViewController.allowsEditing = NO;
    
    
    UINavigationController *navigationController = self.navigationController;

    if ([self isKindOfClass:[ContactSearchViewController class]]) {
    
        ContactSearchViewController *searchSelf = (ContactSearchViewController *)self;
        navigationController    =   searchSelf.searchMasterController.navigationController;
    }
    
    [navigationController pushViewController:contactViewController animated:YES];
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    cell.imageView.layer.cornerRadius = 30.0;
}

#pragma mark -
#pragma mark RightBarButtonAction callback

- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action {
    
    [self showSpinner];
}

- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response {
    
    [self hideSpinner];
    
    PCAPIQueryResponse *APIResponse = (PCAPIQueryResponse *)response;
    
    switch (APIResponse.urlResponse.statusCode) {
        case PCAPIResponseCodeOK: {
            
            switch (action) {
                case accountInfoRightBarAction: {
                    
                    PCAccountInfo *accountInfo = (PCAccountInfo *)result;
                    [self showOKAlertWithTitle:[self stringForRightBarAction:action]
                                    andMessage:[accountInfo humanReadableAccountInfo]
                              attributedString:nil
                                      okAction:nil];
                }
                    break;
                default:
                    break;
            }
            
        }
            break;
        case PCAPIResponseCodeUnauthorized:
            // API call failed because Session is no longer valid.
            [self showUnAuthorizedAlert];
            break;
        case PCAPIResponseCodeServiceUnavailable:
            // API call failed because server unavailable.
            [self showServerUnavailableAlert];
            break;
        default:
            // API call failed due to other network error.
            [self showOperationFailedAlert:[self stringForRightBarAction:action]];
            break;
    }
}

#pragma mark -
#pragma mark CustomCell delegate methods
// CustomCell delegate called when user will do long press in cell.
- (void)tableViewCellDidLongPressed:(UITableViewCell *)cell {

    NSIndexPath *indexPath = [self.tableView indexPathForCell:cell];
    
    PCContact *selectedContact = [self.dataSource objectAtIndex:indexPath.row];
    [self showActionsForSelectedContact:selectedContact];
}

#pragma mark -
#pragma Dealloc

- (void)dealloc {

    for (ContactTableViewCell *cell in self.tableView.visibleCells) {
        
        [cell.avatarImageView.api cancelAPI];
    }
}

@end
