//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <XCTest/XCTest.h>
#import "PCTestConstants.h"

@interface PCGetPlaylistAPITests : XCTestCase

@end

@implementation PCGetPlaylistAPITests

- (void)setUp {
    [super setUp];
    // Put setup code here. This method is called before the invocation of each test method in the class.
}

- (void)tearDown {
    // Put teardown code here. This method is called after the invocation of each test method in the class.
    [super tearDown];
}

- (NSString*)valueBasedOnPlaylistType:(playlistType)playlistType {
    
    NSString *playlistTypeValue;
    
    switch (playlistType) {
        case playlistTypeMusic:
            playlistTypeValue = @"music";
            break;
        case playlistTypeImage:
            playlistTypeValue = @"image";
            break;
        case playlistTypeVideo:
           playlistTypeValue = @"video";
            break;
        case playlistTypeImageVideo:
            playlistTypeValue = @"image-video";
            break;
         case playlistTypeUnknown:
            break;
    }
    return playlistTypeValue;
    
}


/*!
 * Test Case: auth token is a required parameter. SDK should throw an exception in case auth token is nil.
 */
- (void)testNilAuthToken {
    // If User is not logged in, getPlaylist api should throw an exception.
    if (![[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {}],@"Get playlist API with nil access token");
        
    }
}

/*!
 * Test Case: Whenever PCGetPlaylistAPIParams is allocated default value of start needs to be 1. Test fails otherwise.
 */
- (void)testDefaultParameterValue {
    
    PCGetPlaylistAPIParams *param = [[PCGetPlaylistAPIParams alloc] init];
    XCTAssertEqual(param.start, 1);
    
}

/*!
 * Test Case: If start is specified, count is required to get playlist response, . otherwise test will fail.
 */

- (void)testManadatoryCountWhenStartSpecified {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.start = 2;
        params.sort = @"asc";
        params.type = playlistTypeMusic;
        
        XCTAssertThrows([[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {}],@"Get playlist API throw exception without count");
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case: If response code is 200, Playlist array contains zero or greater than zero value. Also playlist object has to be of class PCPlaylist type. In all other cases (error), playlist has to be returned nil.
 */
- (void)testToGetAllPlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
         XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];

        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                }
                    break;
                    
                default:
                      XCTAssertNil(playlist,@"Playlist array return nil");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
    
}

/*!
 * Test Case: If response code is 200 ,Playlist array contains zero or greater than zero value. All playlist object has to be of class PCPlaylist type and contains only music type of playlist. In all other cases (error), playlist has to be returned nil.
 */

- (void)testToGetMusicTypePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result of music type"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeMusic];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    
                    for (PCPlaylist* playlistObject in playlist) {
                        if (![playlistObject.type isEqualToString:[self valueBasedOnPlaylistType:playlistTypeMusic]]) {
                            XCTFail(@"File is not music type");
                        }
                    }
                    
                }
                    break;
                    
                default:
                    XCTAssertNil(playlist,@"Playlist array return nil");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
         XCTFail(@"Test Failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200 ,Playlist array contains zero or greater than zero value. All playlist object has to be of class PCPlaylist type and contains only image type of playlist. In all other cases (error), playlist has to be returned nil.
 */

- (void)testToGetImageTypePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result of image type"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeImage];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    for (PCPlaylist* playlistObject in playlist) {
                        if (![playlistObject.type isEqualToString:[self valueBasedOnPlaylistType:playlistTypeImage]]) {
                            XCTFail(@"File is not image type");
                        }
                    }
                }
                    break;
                    
                default:
                    XCTAssertNil(playlist,@"Playlist array return nil");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200 ,Playlist array contains zero or greater than zero value. All playlist object has to be of class PCPlaylist type and contains only video type of playlist. In all other cases (error), playlist has to be returned nil.
 */

- (void)testToGetVideoTypePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result of video type"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeVideo];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    for (PCPlaylist* playlistObject in playlist) {
                        if (![playlistObject.type isEqualToString:[self valueBasedOnPlaylistType:playlistTypeVideo]]) {
                            XCTFail(@"File is not video type");
                        }
                    }
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

/*!
 * Test Case: If response code is 200 ,Playlist array contains zero or greater than zero value. All playlist object has to be of class PCPlaylist type and contains only image-video type of playlist. In all other cases (error), playlist has to be returned nil.
 */

- (void)testToGetImageVideoTypePlaylist {
    
    if ([[PCAPISDK sharedInstance] isUserLoggedIn]) {
        
        XCTestExpectation *expectation = [self expectationWithDescription:@"Get playlist API Result of video type"];
        
        PCGetPlaylistAPIParams *params = [[PCGetPlaylistAPIParams alloc] init];
        params.type = [self valueBasedOnPlaylistType:playlistTypeImageVideo];
        
        [[PCAPISDK sharedInstance] getPlaylist:params withCompletionHandler:^(NSArray *playlist, PCAPIQueryResponse *response) {
            
            switch (response.urlResponse.statusCode) {
                case PCAPIResponseCodeOK: {
                    XCTAssertGreaterThanOrEqual([playlist count], 0);
                    for (PCPlaylist* playlistObject in playlist) {
                        if (![playlistObject.type isEqualToString:[self valueBasedOnPlaylistType:playlistTypeImageVideo]]) {
                            XCTFail(@"File is not image video type");
                        }
                    }
                }
                    break;
                    
                default:
                    XCTFail(@"Test could not be completed");
                    break;
            }
            [expectation fulfill];
            
        }];
        [self waitForExpectationsWithTimeout:ExpectationTimeOut handler:^(NSError *error) {
            
            if (error) {
                NSLog(@"Timeout Error: %@", error);
            }
        }];
        
    } else {
        
        XCTFail(@"Test Failed due to nil access token");
        
    }
}

@end
