//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import "PCFile+GridCell.h"
#import "ImageDownloadView.h"
#import "PCFile+Info.h"


@implementation PCFile (GridCell)

// Below method set default pic to imageView , when contentToken is unavailable for this file.
- (UIImage *)defaultPic {
    
    if ([[self file] boolValue]) {
        
        PCFile *file = (PCFile *)self;
        
        UIImage *defaultImage = [UIImage imageNamed:@"DefaultPic"];
        
        if (file.fileType == imageFileType) {
            defaultImage = [UIImage imageNamed:@"DefaultPic"];
        } else if (file.fileType == videoFileType) {
            defaultImage = [UIImage imageNamed:@"VideoDefault"];
        } else if (file.fileType == musicFileType) {
            defaultImage = [UIImage imageNamed:@"MusicDefault"];
        }
        return defaultImage;
        
    } else {
        
        return [UIImage imageNamed:@"Folder"];
    }
}

// Below method set thumbnail image to an image view. This method checks for cached image, if not found it downloads image using the content token stored in PCFile. If this fails a default image is set for the image view.
- (void)setThumbnailForImageView:(ImageDownloadView *)imageView withCompletionHandler:(void (^)(BOOL didSetImage))completion {
    
    if ([[self file] boolValue]) {
        
        PCFile *file = (PCFile *)self;
        imageView.imageID = file.contentToken; // imageID needs to be set to encounter cell reusability feature.
        
        UIImage *thumbnailImage = [UIImage imageWithContentsOfFile:[self thumbnailCachePath]];
        
        
        if (thumbnailImage) {
            
            imageView.image = thumbnailImage;
            completion(YES);
        } else {
            
            imageView.image = nil;
            __weak PCFile *blocksafeSelf = self;
            
            /*!
             @Description-This method name is downloadThumbnail, Call this method to download thumbnail image of any file.
             @ Parameters - (PCThumbnailAPIParams object) attributes such as th height of the image ,tw width of the image ,contentToken of the resource.
             @CompletionHandler-This completion block gets called on the completion of thumbnail fetch process. PCAPIQueryResponse stores the location of downloaded thumbnail, Return type are (PCAPIQueryResponse object).
             */

            
          PCAPI *thumbnailAPI = [file downloadThumbnailWithCompletionHandler:^(PCAPIQueryResponse *response) {
              
                NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                if (fileData) {
                    [fileData writeToFile:[blocksafeSelf thumbnailCachePath] atomically:YES]; // Writting image data in thumbnailCachePath.
                }
                
                if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                    
                    
                    if (fileData) {
                        
                        UIImage *thumbImage = [UIImage imageWithContentsOfFile:[blocksafeSelf thumbnailCachePath]];
                        if (thumbImage) {
                            
                            imageView.image = thumbImage;
                            completion(YES);
                        } else {
                            
                            imageView.image = [blocksafeSelf defaultPic];
                            completion(YES);
                        }
                        
                    } else {
                        
                        imageView.image = [blocksafeSelf defaultPic];
                        completion(YES);
                    }
                } else {
                    
                    completion(NO);
                }
            }];
            
            imageView.api = thumbnailAPI;
        }
        
    } else {
        
        imageView.imageID = nil;
        imageView.image = [self defaultPic]; // Assigning default pic when contentToken unavailable for PCFile object.
        completion(YES);
    }
}

// Below method set a full resolution image to an image view. This method checks for cached image, if not found it downloads image using the image url stored in PCFile. If this fails a default image is set for the image view.

- (void)setFullResolutionImageView:(ImageDownloadView *)imageView withProgressHandler:(void (^)(float progress))progress andCompletionHandler:(void (^)(BOOL didSetImage))completion {
    
    if (self.fileType == imageFileType) {
        
        PCFile *file = (PCFile *)self;
        imageView.imageID = file.contentToken; // imageID needs to be set to encounter cell reusability feature
        
        UIImage *image = [[UIImage alloc] initWithContentsOfFile:[self fileCachePath]];
        
        if (image) {
            
            imageView.image = image;
            completion(YES);
        } else {
            
            imageView.image = nil;
            // Intialize the PCFilesAPIParams object.
            PCFilesAPIParams *params = [[PCFilesAPIParams alloc] init];
            params.path = self.fullPath;
            
            __weak PCFile *blocksafeSelf = self;
            
            /*
             @Description :
             This API is used to download a file. Once the download process is succeed/failed result will come with completion block.
             
             @Parameters :
             PCFilesAPIParams:
             * This class stores all the parameters required to download a file. It contains attributes such as path.
             * For providing access token to SDK user have to logged In.
             * path is required parameter to call this API.
             * If user will not provide required parameter, API will throw an NSInvalidArgumentException and user have to handle it.
             * More information about parameters you will find in PCFilesAPIParams.h class.
            
             @CompletionHandler :
             This completion block gets called once the download process is succeed/failed. completion block contains below parameter.
             PCAPIQueryResponse : Stores all the server response and error for this API. dowloaded result comes in downloadedURL attribute.The file is downloaded to a temporary path and the file will be removed once the api which downloaded the file is released. So move the file to a different location in completion callback.
              progress : It will return totalBytesDownloaded and totalBytesToBeDownloaded.
             
             @To download a file, you must take the following steps:
             * Include VerizonCloudAPI.framework header file in your class.
             #import <VerizonCloudAPI/VerizonCloudAPI.h>
             * Initalize the PCFilesAPIParams class and provide the required parameter.
             * Initalize the PCAPISDK object and call the  downloadFile:progress:withCompletionHandler: API.
             * Perform the  downloadFile:progress:withCompletionHandler: API call using the code sample below.
             * Get the result (succeed or failed) in completion block.
             * If session is no longer valid , call the logout API which remove Access token stored in keychain. Again call authenticate:fromViewController:withCompletionHandler: API to get Access token.
             */
            
            // Initilize the PCAPISDK and call the API.
            
            
            PCAPI *filesAPI = [[PCAPISDK sharedInstance] downloadFile:params
                                                             progress:^(long long totalBytesDownloaded, long long totalBytesToBeDownloaded) {
                                                                 // Get the Progress in this block.
                                                                 progress((float)totalBytesDownloaded/[[self size] floatValue]);
                                                                 
                                                             } withCompletionHandler:^(PCAPIQueryResponse *response) {
                                                                 // Get the result in this block.

                                                                 NSData *fileData = [NSData dataWithContentsOfURL:response.downloadedURL];
                                                                 
                                                                 if (fileData) {
                                                                     
                                                                     [fileData writeToFile:[self fileCachePath] atomically:YES];
                                                                 }
                                                                 
                                                                 if ([imageView.imageID isEqualToString:blocksafeSelf.contentToken]) { // cell is still representing same resource
                                                                     
                                                                     
                                                                     if (fileData) {
                                                                         
                                                                         UIImage *fullResImage = [UIImage imageWithContentsOfFile:[blocksafeSelf fileCachePath]];
                                                                         if (fullResImage) {
                                                                             
                                                                             imageView.image = fullResImage;
                                                                             completion(YES);
                                                                         } else {
                                                                             
                                                                             imageView.image = [blocksafeSelf defaultPic];
                                                                             completion(YES);
                                                                         }
                                                                         
                                                                     } else {
                                                                         
                                                                         imageView.image = [blocksafeSelf defaultPic];
                                                                         completion(YES);
                                                                     }
                                                                 } else {
                                                                     
                                                                     completion(NO);
                                                                 }
                                                             }];
            
            imageView.api = filesAPI;
            
        }
    } else {
        
        imageView.imageID = nil;
        imageView.image = [self defaultPic];
        completion(YES);
    }
}



@end
