//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

@interface UIViewController (Alert)

/*!
 * show operation failed alert.
 */
- (void)showOperationFailedAlert:(NSString *)operationName;

/*!
 * show server unavailable alert.
 */
- (void)showServerUnavailableAlert;

/*!
 * show Session is no longer valid alert.
 */
- (void)showUnAuthorizedAlert;

/*!
 * show email not configured alert.
 */
- (void)showEmailConfigurationAlert;

/*!
 * call this method to show an alert with 'OK' button.
 * \params title Title to be shown.
 * \params message message to be shown.
 * \params handle block to be called on pressing OK button.
 */
- (void)showOKAlertWithTitle:(NSString *)title
                  andMessage:(NSString *)message
            attributedString:(NSAttributedString *)attributedString
                    okAction:(void (^)(UIAlertAction *action))handle;

/*!
 * call this method to show an alert with 'OK' and 'Cancel' button.
 * \params title Title to be shown.
 * \params message message to be shown.
 * \params OKhandle block to be called on pressing OK button.
 * \params Cancelhandle block to be called on pressing Cancel button.
 */
- (void)showOKCancelAlertWithTitle:(NSString *)title
                        andMessage:(NSString *)message
                          okAction:(void (^)(UIAlertAction *action))OKhandle
                      cancelAction:(void (^)(UIAlertAction *action))Cancelhandle;

/*!
 * call this method to show an alert with 'Send' and 'Cancel' button.
 * \params title Title to be shown.
 * \params message message to be shown.
 * \params sendHandle block to be called on pressing send button.
 * \params Cancelhandle block to be called on pressing Cancel button.
 */
- (void)showOKSendAlertWithTitle:(NSString *)title
                      andMessage:(NSString *)message
                        okAction:(void (^)(UIAlertAction *action))sendHandle
                    cancelAction:(void (^)(UIAlertAction *action))Cancelhandle;

/*!
 * call this method to show an alert with UIAlertAction.
 * \params alertController UIAlertController instance.
 * \params title Title to be shown.
 * \params style UIAlertActionStyle to be shown.
 * \params handler block to be called to handle action.
 */
- (void)alertController:(UIAlertController *)alertController
     addActionWithTitle:(NSString *)title
                  style:(UIAlertActionStyle)style
              andAction:(void (^)(UIAlertAction *action))handler;

/*!
 * call this method to show an alert with text input.
 * \params alertController UIAlertController instance.
 * \params title Title to be shown.
 * \params handler block to be called to handle action.
 */
- (void)alertController:(UIAlertController *)alertController
showTextInputAlertWithTitle:(NSString *)title
 andOKCompletionHandler:(void (^)(UIAlertAction *action))handler;


@end
