//Copyright © 2015-2016, Verizon and/or its Licensors. All rights reserved.

#import <UIKit/UIKit.h>

typedef enum {

    accountInfoRightBarAction,
    searchRightBarAction,
#ifdef KEPLER_FILE_BROWSER
    createFolderRightBarAction,
    uploadFileRightBarAction,
    emptyTrashRightBarAction,
    photoMediaRightBarAction,
    videoMediaRightBarAction,
    audioMediaRightBarAction,
    docMediaRightBarAction,
    accessTokenRightBarAction,
#endif
#ifdef HUBBLE_MEDIA_BROWSER
    createPlaylistRightBarAction,
    createAlbumRightBarAction,
    shareRightBarAction,
#endif
    signOutRightBarAction,
    refreshAccountBarAction
} rightBarAction;

/*!
 * This is a UIViewController category with additional methods like CreateFolder, AccountInfo, Search, SignOut etc. These methods are triggered on the click of a right bar button. Any viewController which needs to implement the right bar button with these actions can include this category.
 */

@interface UIViewController (RightBarButtonAction)

/*!
 * call this method to get string for RightBarAction passed.
 * @param action rightBarAction on which action needs to be performed.
 */
- (NSString *)stringForRightBarAction:(rightBarAction)action;

/*!
 * call this method to include a button on the right side of the naviation bar. Clicking on this button will pops up an action sheet with options like SignOut, Search and Accountinfo etc.
 */
- (void)addRightBarButtonItemWithActions:(id) firstAction, ... NS_REQUIRES_NIL_TERMINATION;

/*!
 * call this method if any of the actions need to be executed by silently.
 */
- (void)executeAction:(rightBarAction)action silent:(BOOL)silent;


/*!
 * Implement this action in classes which consume this category if they needs to pass any data inorder to perform a particular action. For eg a root path in Folder create operation.
 */
- (id)dataSourceForRightBarButtonAction:(rightBarAction)action;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement custom actions which needs to be implemented before performing a particular action. For eg. start a spinner.
 * @param action the action that is going to be executed.
 */
- (void)didBeginDoingRightBarButtonAction:(rightBarAction)action;

/*!
 * Implement this method in the viewController consuming this category class. This method can implement custom actions which needs to be implemented after performing a particular action. For eg. stop a spinner.
 * @param action the action that is being executed.
 * @param result result of action, if any.
 * @param response response of the action, if any.
 */
- (void)didFinishDoingRightBarButtonAction:(rightBarAction)action :(id)result :(id)response;

@end
